#!/usr/bin/perl
#---------------------------------------------------------------------
#/* Copyright (C) 2000-2006 MySQL AB
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# Version 1.1
#
######################################################################
# Original Author: Serge Kozlov (skozlov@mysql.com)
# Original Date: 2006-11-16
######################################################################
use DBI;
use strict;
use Getopt::Long; 
Getopt::Long::Configure ("bundling_override");

####################################
# Global variables
####################################

# Hash for command line options
our %cmd_options = ();
our @queries = ();
our $dbh;
our $my_host='localhost'; 
our $my_port='3306';
our $my_user='root';
our $my_pass='';
our $my_db='test';


sub main;
sub Command_Line_Args();
sub ConnectToDatabase();
sub DisconnectFromDatabase();
sub ReadQueryFile();
sub RunSingleQuery($);
sub RunQuery();

######################################################################
#
#  Main program
#
######################################################################

main();

sub main()
{
  # Collect command line arguments to %cmd_options
  Command_Line_Args();
  # Read test file file and fill out %testVars and @testOrder
  ReadQueryFile();
  ConnectToDatabase();
  RunQuery();
  DisconnectFromDatabase();
}

######################################################################
#
# Process command line options
#
######################################################################

sub Command_Line_Args() 
{
  my $m_host;
  my $m_port;
  my $m_user;
  my $m_pass;
  my $m_db;
  if (!GetOptions("help|h"=>\$cmd_options{help},
                "queryFile|q:s"=>\$cmd_options{queryFile},
                "myParam|p:s"=>\$cmd_options{myParam}))
  {
  }
  ($m_host, $m_port, $m_user, $m_pass, $m_db) = split(/\:/, $cmd_options{myParam});
  $my_host = $m_host unless ($m_host eq '');
  $my_port = $m_port unless ($m_port eq '');
  $my_user = $m_user unless ($m_user eq '');
  $my_pass = $m_pass unless ($m_pass eq '');
  $my_db = $m_db unless ($m_db eq '');
}

######################################################################
#
#  Connect to database
#
######################################################################
sub ConnectToDatabase ()
{
   $dbh = DBI->connect("dbi:mysql:database=mysql;host=$my_host;port=$my_port","$my_user", "$my_pass")
     or die "Can't connect to MySQL process! Error: $DBI::errstr\n";
  print "Connected to MySQLD.\n-------------------------\n";
}

sub DisconnectFromDatabase ()
{
  $dbh->disconnect
    or warn " Disconnection failed: $DBI::errstr\n";
  print "-------------------------\nDisconnected from MySQLD.\n";
}

######################################################################
#
# Read data from test file defined by --testFile argument
#
######################################################################

sub ReadQueryFile()
{
  # Local variables
  my $m_string;
  my $m_queryfile = $cmd_options{queryFile};
  open F, $m_queryfile or die "Error: file ".$m_queryfile." not found";
  # Reading testfile
  while (<F>) 
  {
    $m_string = $_;
    # Remove comments
    $m_string =~ s/\#.*$//;
    $m_string =~ s/\s+$//;
    $m_string =~ s/^ +//g;
    push @queries, $m_string;
  }
  close F;
}

sub RunSingleQuery($)
{
  my $m_query = shift;
  if ($m_query ne "") {
    print $m_query."\n";
    my $m_sth = $dbh->prepare($m_query)
      or die "Error: ", $dbh->errstr;
    $m_sth->execute();
    $m_sth->finish();
  }
}

sub RunQuery()
{
  my $m_counter;
  my $m_sth;
  my %m_vars = ();
  my $m_line;
  my $m_loop_var;
  my $m_loop_counter_start;
  my $m_loop_counter_end;
  my $m_var;
  for ($m_counter = 0; $m_counter < scalar(@queries); $m_counter++) 
  {
    $m_line = $queries[$m_counter];   
    if ($m_line =~ m/^ *let +\$(.+?) *= *(.+?)[;]*$/g) 
    {
      $m_vars{$1} = $2;
    }
    elsif ($m_line =~ m/^ *while *\(\$(.+?)\) *\{/g)
    {
      $m_loop_var = $1;
      $m_loop_counter_start = $m_counter;
      if ($m_vars{$m_loop_var} <= 0) 
      {
        $m_counter = $m_loop_counter_end;
      }
    }
    elsif ($m_line =~ m/^ *\}/g)
    {
      $m_loop_counter_end = $m_counter;
      $m_counter = $m_loop_counter_start-1;
    }
    elsif ($m_line =~ m/^ *inc +\$(.+?)[;]*$/g)
    {
      $m_vars{$1}++;
    }
    elsif ($m_line =~ m/^ *dec +\$(.+?)[;]*$/g)
    {
      $m_vars{$1}--;
    }
    else
    {
      foreach $m_var (keys %m_vars) {
        $m_line =~ s /\$$m_var/$m_vars{$m_var}/ig;
      }
      RunSingleQuery($m_line);
    }
  }
}